@@ -1,5 +1,5 @@
 package Juju::Environment;
-$Juju::Environment::VERSION = '1.7';
+$Juju::Environment::VERSION = '1.8';
 # ABSTRACT: Exposed juju api environment
 
 
@@ -8,27 +8,31 @@ use warnings;
 use JSON::PP;
 use YAML::Tiny qw(Dump);
 use Data::Validate::Type qw(:boolean_tests);
-use Params::Validate qw(:all);
+use Function::Parameters qw(:strict);
 use Juju::Util;
-use parent 'Juju::RPC';
+use Moo;
+use namespace::clean;
+with 'Juju::RPC';
 
 
-use Class::Tiny qw(password is_authenticated), {
-    endpoint => sub {'wss://localhost:17070'},
-    username => sub {'user-admin'},
-    Jobs     => sub {
+has password => (is => 'ro', required => 1);
+has is_authenticated => (is => 'rw', lazy => 1);
+has endpoint => (is => 'ro', default => sub {'wss://localhost:17070'});
+has username => (is => 'ro', default => sub {'user-admin'});
+has Jobs => (
+    is      => 'ro',
+    default => sub {
         {   HostUnits     => 'JobHostUnits',
             ManageEnviron => 'JobManageEnviron',
             ManageState   => 'JobManageSate'
         };
-    },
-    util   => Juju::Util->new
-};
+    }
+);
+has util => (is => 'ro', default => sub { Juju::Util->new });
 
 
 
-sub _prepare_constraints {
-    my ($self, $constraints) = @_;
+method _prepare_constraints ($constraints) {
     foreach my $key (keys %{$constraints}) {
         if ($key =~ /^(cpu-cores|cpu-power|mem|root-disk)/) {
             $constraints->{k} = int($constraints->{k});
@@ -38,10 +42,7 @@ sub _prepare_constraints {
 }
 
 
-sub login {
-    my $self = shift;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-
+method login {
     my $params = {
         "Type"      => "Admin",
         "Request"   => "Login",
@@ -61,18 +62,14 @@ sub login {
 
 
 
-
-sub reconnect {
-    my $self = shift;
+method reconnect {
     $self->close;
     $self->login;
     $self->request_id = 1;
 }
 
 
-sub environment_info {
-    my $self = shift;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method environment_info($cb = undef) {
     my $params = {"Type" => "Client", "Request" => "EnvironmentInfo"};
 
     # block
@@ -82,9 +79,8 @@ sub environment_info {
     return $self->call($params, $cb);
 }
 
-sub environment_uuid {
-    my $self = shift;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+
+method environment_uuid ($cb = undef) {
     my $params = {"Type" => "Client", "Request" => "EnvironmentUUID"};
 
     # block
@@ -96,9 +92,7 @@ sub environment_uuid {
 
 
 
-sub environment_unset {
-    my ($self, $items) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method environment_unset ($items, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "EnvironmentUnset",
@@ -112,9 +106,8 @@ sub environment_unset {
     return $self->call($params, $cb);
 }
 
-sub find_tools {
-    my ($self, $major, $minor, $series, $arch) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+
+method find_tools ($major, $minor, $series, $arch, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "EnvironmentUnset",
@@ -134,9 +127,8 @@ sub find_tools {
 }
 
 
-sub agent_version {
-    my $self = shift;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+
+method agent_version ($cb = undef) {
     my $params = {"Type" => "Client", "Request" => "AgentVersion"};
 
     # block
@@ -146,9 +138,8 @@ sub agent_version {
     return $self->call($params, $cb);
 }
 
-sub abort_current_upgrade {
-    my $self = shift;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+
+method abort_current_upgrade ($cb = undef) {
     my $params = {"Type" => "Client", "Request" => "AbortCurrentUpgrade"};
 
     # block
@@ -160,11 +151,9 @@ sub abort_current_upgrade {
 
 
 
-sub status {
-    my $self   = shift;
-    my $cb     = ref $_[-1] eq 'CODE' ? pop : undef;
+method status ($cb = undef) {
     my $params = {
-        "Type"   => "Client",
+        "Type"    => "Client",
         "Request" => "FullStatus"
     };
 
@@ -175,11 +164,10 @@ sub status {
     return $self->call($params, $cb);
 }
 
-sub client_api_host_ports {
-    my $self   = shift;
-    my $cb     = ref $_[-1] eq 'CODE' ? pop : undef;
+
+method client_api_host_ports ($cb = undef) {
     my $params = {
-        "Type"   => "Client",
+        "Type"    => "Client",
         "Request" => "APIHostPorts"
     };
 
@@ -192,9 +180,7 @@ sub client_api_host_ports {
 
 
 
-sub get_watcher {
-    my $self = shift;
-    my $cb     = ref $_[-1] eq 'CODE' ? pop : undef;
+method get_watcher ($cb = undef) {
     my $params = {"Type" => "Client", "Request" => "WatchAll"};
 
     # block
@@ -205,9 +191,7 @@ sub get_watcher {
 }
 
 
-sub get_watched_tasks {
-    my ($self, $watcher_id) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method get_watched_tasks ($watcher_id, $cb = undef) {
     die "Unable to run synchronously, provide a callback" unless $cb;
 
     my $params =
@@ -219,9 +203,7 @@ sub get_watched_tasks {
 
 
 
-sub add_charm {
-    my ($self, $charm_url) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method add_charm ($charm_url, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "AddCharm",
@@ -236,9 +218,7 @@ sub add_charm {
 }
 
 
-sub get_charm {
-    my ($self, $charm_url) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method get_charm ($charm_url, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "CharmInfo",
@@ -253,9 +233,7 @@ sub get_charm {
 }
 
 
-sub get_environment_constraints {
-    my $self   = shift;
-    my $cb     = ref $_[-1] eq 'CODE' ? pop : undef;
+method get_environment_constraints ($cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "GetEnvironmentConstraints"
@@ -270,9 +248,7 @@ sub get_environment_constraints {
 }
 
 
-sub set_environment_constraints {
-    my ($self, $constraints) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method set_environment_constraints (ArrayRef $constraints, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "SetEnvironmentConstraints",
@@ -287,9 +263,7 @@ sub set_environment_constraints {
 }
 
 
-sub environment_get {
-    my $self   = shift;
-    my $cb     = ref $_[-1] eq 'CODE' ? pop : undef;
+method environment_get ($cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "EnvironmentGet"
@@ -303,9 +277,7 @@ sub environment_get {
 }
 
 
-sub environment_set {
-    my ($self, $config) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method environment_set ($config, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "EnvironmentSet",
@@ -320,12 +292,7 @@ sub environment_set {
 }
 
 
-sub add_machine {
-    my $self = shift;
-    my ($series, $constraints, $machine_spec, $parent_id, $container_type) =
-      validate_pos(@_, {default => 'trusty'}, 0, 0, 0, 0);
-
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method add_machine ($series, $constraints = undef, $machine_spec = undef, $parent_id = undef, $container_type = undef, $cb = undef) {
     my $params = {
         "Series"        => $series,
         "Jobs"          => [$self->Jobs->{HostUnits}],
@@ -352,9 +319,7 @@ sub add_machine {
 }
 
 
-sub add_machines {
-    my ($self, $machines) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method add_machines ($machines, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "AddMachines",
@@ -370,9 +335,7 @@ sub add_machines {
 
 
 
-sub destroy_environment {
-    my $self   = shift;
-    my $cb     = ref $_[-1] eq 'CODE' ? pop : undef;
+method destroy_environment ($cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "DestroyEnvironment"
@@ -387,10 +350,7 @@ sub destroy_environment {
 }
 
 
-sub destroy_machines {
-    my ($self, $machine_ids, $force) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-
+method destroy_machines ($machine_ids, $force = 0, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "DestroyMachines",
@@ -410,9 +370,7 @@ sub destroy_machines {
 }
 
 
-sub add_relation {
-    my ($self, $endpoint_a, $endpoint_b) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method add_relation ($endpoint_a, $endpoint_b, $cb = undef) {
     my $params = {
         'Type'    => 'Client',
         'Request' => 'AddRelation',
@@ -427,9 +385,7 @@ sub add_relation {
 }
 
 
-sub destroy_relation {
-    my ($self, $endpoint_a, $endpoint_b) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method destroy_relation ($endpoint_a, $endpoint_b, $cb = undef) {
     my $params = {
         'Type'    => 'Client',
         'Request' => 'DestroyRelation',
@@ -444,58 +400,39 @@ sub destroy_relation {
 }
 
 
-sub deploy {
-    my $self = shift;
-    my %p    = validate(
-        @_,
-        {   charm        => {type    => SCALAR},
-            service_name => {type    => SCALAR},
-            num_units    => {default => 1},
-            config_yaml  => {default => ""},
-            constraints  => {default => +{}, type => HASHREF},
-            machine_spec => {default => ""},
-            cb           => {default => undef}
-        }
-    );
-
+method deploy ($charm, $service_name, $num_units = 1, $config_yaml = "", $constraints = "", $machine_spec = "", $cb = undef) {
     my $params = {
         Type    => "Client",
         Request => "ServiceDeploy",
-        Params  => {ServiceName => $p{service_name}}
+        Params  => {ServiceName => $service_name}
     };
 
     # Check for series format
-    my (@charm_args) = $p{charm} =~ /(\w+)\/(\w+)/i;
+    my (@charm_args) = $charm =~ /(\w+)\/(\w+)/i;
     my $_charm_url = undef;
     if (scalar @charm_args == 2) {
         $_charm_url = $self->util->query_cs($charm_args[1], $charm_args[0]);
     }
     else {
-        $_charm_url = $self->util->query_cs($p{charm});
+        $_charm_url = $self->util->query_cs($charm);
     }
 
     $params->{Params}->{CharmUrl}   = $_charm_url->{charm}->{url};
-    $params->{Params}->{NumUnits}   = $p{num_units};
-    $params->{Params}->{ConfigYAML} = $p{config_yaml};
+    $params->{Params}->{NumUnits}   = $num_units;
+    $params->{Params}->{ConfigYAML} = $config_yaml;
     $params->{Params}->{Constraints} =
-      $self->_prepare_constraints($p{constraints});
-    $params->{Params}->{ToMachineSpec} = "$p{machine_spec}";
+      $self->_prepare_constraints($constraints);
+    $params->{Params}->{ToMachineSpec} = "$machine_spec";
 
     # block
-    return $self->call($params) unless $p{cb};
+    return $self->call($params) unless $cb;
 
     # non-block
-    return $self->call($params, $p{cb});
+    return $self->call($params, $cb);
 }
 
 
-sub service_set {
-    my $self = shift;
-    my ($service_name, $config) =
-      validate_pos(@_, {type => SCALAR}, {type => HASHREF, optional => 1});
-
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-
+method service_set ($service_name, $config = undef, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "ServiceSet",
@@ -513,18 +450,15 @@ sub service_set {
 }
 
 
-sub unset_config {
-    my ($self, $service_name, $config_keys) = @_;
-    my $cb     = ref $_[-1] eq 'CODE' ? pop : undef;
-
-    my $params = 
-        {   "Type"    => "Client",
-            "Request" => "ServiceUnset",
-            "Params"  => {
-                "ServiceName" => $service_name,
-                "Options"     => $config_keys
-            }
-        };
+method unset_config ($service_name, $config_keys, $cb = undef) {
+    my $params = {
+        "Type"    => "Client",
+        "Request" => "ServiceUnset",
+        "Params"  => {
+            "ServiceName" => $service_name,
+            "Options"     => $config_keys
+        }
+    };
 
     # block
     return $self->call($params) unless $cb;
@@ -534,11 +468,7 @@ sub unset_config {
 }
 
 
-sub set_charm {
-    my ($self, $service_name, $charm_url, $force) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-
-    $force = 0 unless $force;
+method set_charm ($service_name, $charm_url, $force = 0, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "ServiceSetCharm",
@@ -557,9 +487,7 @@ sub set_charm {
 }
 
 
-sub service_get {
-    my ($self, $service_name) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method service_get ($service_name, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "ServiceGet",
@@ -574,19 +502,14 @@ sub service_get {
 }
 
 
-sub get_config {
-    my ($self, $service_name) = @_;
-    my $cb     = ref $_[-1] eq 'CODE' ? pop : undef;
-
+method get_config ($service_name, $cb = undef) {
     my $svc = $self->service_get($service_name);
     return $svc->{Config} unless $cb;
     return $cb->($svc->{Config});
 }
 
 
-sub get_service_constraints {
-    my ($self, $service_name) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method get_service_constraints ($service_name, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "GetServiceConstraints",
@@ -601,9 +524,7 @@ sub get_service_constraints {
 }
 
 
-sub set_service_constraints {
-    my ($self, $service_name, $constraints) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method set_service_constraints ($service_name, $constraints, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "SetServiceConstraints",
@@ -620,9 +541,8 @@ sub set_service_constraints {
     return $self->call($params, $cb);
 }
 
-sub share_environment {
-    my ($self, $users) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+
+method share_environment ($users, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "ShareEnvironment",
@@ -636,9 +556,8 @@ sub share_environment {
     return $self->call($params, $cb);
 }
 
-sub unshare_environment {
-    my ($self, $users) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+
+method unshare_environment ($users, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "UnshareEnvironment",
@@ -654,9 +573,7 @@ sub unshare_environment {
 
 
 
-sub service_destroy {
-    my ($self, $service_name) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method service_destroy ($service_name, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "ServiceDestroy",
@@ -671,9 +588,7 @@ sub service_destroy {
 }
 
 
-sub service_expose {
-    my ($self, $service_name) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method service_expose ($service_name, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "ServiceExpose",
@@ -689,9 +604,7 @@ sub service_expose {
 
 
 
-sub service_unexpose {
-    my ($self, $service_name) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method service_unexpose ($service_name, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "ServiceUnexpose",
@@ -706,9 +619,7 @@ sub service_unexpose {
 }
 
 
-sub service_charm_relations {
-    my ($self, $service_name) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method service_charm_relations ($service_name, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "ServiceCharmRelations",
@@ -723,11 +634,7 @@ sub service_charm_relations {
 }
 
 
-sub add_service_units {
-    my ($self, $service_name, $num_units) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-
-    $num_units = 1 unless $num_units;
+method add_service_units ($service_name, $num_units = 1, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "AddServiceUnits",
@@ -745,11 +652,7 @@ sub add_service_units {
 }
 
 
-sub add_service_unit {
-    my ($self, $service_name, $machine_spec) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-
-    $machine_spec = 0 unless $machine_spec;
+method add_service_unit ($service_name, $machine_spec = "", $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "AddServiceUnits",
@@ -771,9 +674,7 @@ sub add_service_unit {
 }
 
 
-sub destroy_service_units {
-    my ($self, $unit_names) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method destroy_service_units ($unit_names, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "DestroyServiceUnits",
@@ -788,11 +689,7 @@ sub destroy_service_units {
 }
 
 
-sub resolved {
-    my ($self, $unit_name, $retry) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-
-    $retry = 0 unless $retry;
+method resolved ($unit_name, $retry = 0, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "Resolved",
@@ -811,43 +708,28 @@ sub resolved {
 
 
 
-sub run {
-    my $self = shift;
-    my %p    = validate(
-        @_,
-        {   command  => {type => SCALAR},
-            timeout  => {type => SCALAR, default => 300},
-            machines => {type => ARRAYREF, optional => 1},
-            services => {type => ARRAYREF, optional => 1, default => +[]},
-            units    => {type => ARRAYREF, optional => 1, default => +[]},
-            cb => {type => CODEREF, optional => 1}
-        }
-    );
+method run ($command, $timeout, $machines = undef, $services = undef, $units = undef, $cb = undef) {
     my $params = {
         Type    => "Client",
         Request => "Run",
         Params  => {
-            Commands => $p{command},
-            Timeout  => $p{timeout},
-            Machines => @{$p{machines}},
-            Services => @{$p{services}},
-            Units    => @{$p{units}}
+            Commands => $command,
+            Timeout  => $timeout,
+            Machines => $machines,
+            Services => $services,
+            Units    => $units
         }
     };
 
     # block
-    return $self->call($params) unless $p{cb};
+    return $self->call($params) unless $cb;
 
     # non-block
-    return $self->call($params, $p{cb});
+    return $self->call($params, $cb);
 }
 
 
-sub run_on_all_machines {
-    my ($self, $command, $timeout) = @_;
-
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-
+method run_on_all_machines ($command, $timeout, $cb = undef) {
     my $params = {
         Type    => "Client",
         Request => "RunOnAllMachines",
@@ -865,9 +747,7 @@ sub run_on_all_machines {
 }
 
 
-sub set_annotations {
-    my ($self, $entity, $entity_type, $annotation) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method set_annotations ($entity, $entity_type, $annotation, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "SetAnnotations",
@@ -885,9 +765,7 @@ sub set_annotations {
 }
 
 
-sub get_annotations {
-    my ($self, $entity, $entity_type) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+method get_annotations ($entity, $entity_type, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "GetAnnotations",
@@ -904,9 +782,8 @@ sub get_annotations {
     return $self->call($params, $cb);
 }
 
-sub private_address {
-    my ($self, $target) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+
+method private_address ($target, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "PrivateAddress",
@@ -920,9 +797,8 @@ sub private_address {
     return $self->call($params, $cb);
 }
 
-sub public_address {
-    my ($self, $target) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+
+method public_address ($target, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "PublicAddress",
@@ -936,9 +812,8 @@ sub public_address {
     return $self->call($params, $cb);
 }
 
-sub service_set_yaml {
-    my ($self, $service, $yaml) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+
+method service_set_yaml ($service, $yaml, $cb = undef) {
     my $params = {
         "Type"    => "Client",
         "Request" => "PublicAddress",
@@ -970,13 +845,14 @@ Juju::Environment - Exposed juju api environment
 
 =head1 VERSION
 
-version 1.7
+version 1.8
 
 =head1 SYNOPSIS
 
   use Juju;
 
-  my $juju = Juju->new(endpoint => 'wss://localhost:17070', password => 's3cr3t');
+  my $juju =
+    Juju->new(endpoint => 'wss://localhost:17070', password => 's3cr3t');
 
 =head1 ATTRIBUTES
 
@@ -1004,6 +880,10 @@ Stores if user has authenticated with juju api server
 
 Supported juju jobs
 
+=head2 util
+
+L<Juju::Util> wrapper
+
 =head1 METHODS
 
 =head2 _prepare_constraints
@@ -1322,12 +1202,16 @@ Second unit endpoint
 
 =head2 deploy
 
-Deploys a charm to service, named parameters are required here:
+Deploys a charm to service
 
     $juju->deploy(
-        charm        => 'mysql',
-        service_name => 'mysql',
-        cb           => sub {
+        'mysql',
+        'mysql',
+        1,
+        undef,
+        undef,
+        undef,
+        sub {
             my $val = shift;
             print Dumper($val) if defined($val->{Error});
         }
